==> python_alist-0.0.11.5.2.tar.gz: python_alist-0.0.11.5.2/alist/cmd/update_115_cookie.py <==
#!/usr/bin/env python3
# encoding: utf-8

__author__ = "ChenyangGao <https://chenyanggao.github.io>"
__all__: list[str] = []
__doc__ = "alist 更新 115 cookie"

if __name__ == "__main__":
    from argparse import ArgumentParser, RawTextHelpFormatter

    parser = ArgumentParser(description=__doc__, formatter_class=RawTextHelpFormatter)
else:
    from .init import subparsers

    parser = subparsers.add_parser("115", description=__doc__)

from enum import Enum
from http.client import HTTPResponse
from json import dumps, load, loads
from urllib.parse import urlencode
from urllib.request import urlopen, Request

from alist import __version__, AlistClient


AppEnum = Enum("AppEnum", "web, android, ios, linux, mac, windows, tv, alipaymini, wechatmini, qandroid")


def get_enum_name(val, cls):
    if isinstance(val, cls):
        return val.name
    try:
        if isinstance(val, str):
            return cls[val].name
    except KeyError:
        pass
    return cls(val).name


def get_qrcode_token() -> dict:
    """获取登录二维码，扫码可用
    GET https://qrcodeapi.115.com/api/1.0/web/1.0/token/

    :return: 扫码相关的信息，比如二维码的 uid
    """
    api = "https://qrcodeapi.115.com/api/1.0/web/1.0/token/"
    return load(urlopen(api))


def get_qrcode_status(payload) -> dict:
    """获取二维码的状态（未扫描、已扫描、已登录、已取消、已过期等）
    GET https://qrcodeapi.115.com/get/status/

    :param payload: 请求的查询参数，取自 `login_qrcode_token` 接口响应，有 3 个
        - uid:  str
        - time: int
        - sign: str

    :return: 二维码目前的扫码状态信息
    """
    api = "https://qrcodeapi.115.com/get/status/?" + urlencode(payload)
    return load(urlopen(api))


def post_qrcode_result(uid: str, app: str = "web") -> dict:
    """获取扫码登录的结果，并且绑定设备，包含 cookie
    POST https://passportapi.115.com/app/1.0/{app}/1.0/login/qrcode/

    :param uid: 二维码的 uid，取自 `login_qrcode_token` 接口响应
    :param app: 扫码绑定的设备，可以是 int、str 或者 AppEnum
        app 目前发现的可用值：
            - 1,  "web",        AppEnum.web
            - 2,  "android",    AppEnum.android
            - 3,  "ios",        AppEnum.ios
            - 4,  "linux",      AppEnum.linux
            - 5,  "mac",        AppEnum.mac
            - 6,  "windows",    AppEnum.windows
            - 7,  "tv",         AppEnum.tv
            - 8,  "alipaymini", AppEnum.alipaymini
            - 9,  "wechatmini", AppEnum.wechatmini
            - 10, "qandroid",   AppEnum.qandroid

    :return: 包含 cookie 的响应
    """
    app = get_enum_name(app, AppEnum)
    payload = {"app": app, "account": uid}
    api = "https://passportapi.115.com/app/1.0/%s/1.0/login/qrcode/" % app
    return load(urlopen(Request(api, data=urlencode(payload).encode("utf-8"), method="POST")))


def get_qrcode(uid: str) -> HTTPResponse:
    """获取二维码图片（注意不是链接）
    :return: 一个文件对象，可以读取
    """
    url = "https://qrcodeapi.115.com/api/1.0/mac/1.0/qrcode?uid=%s" % uid
    return urlopen(url)


def login_with_qrcode(
    app: str = "web", 
    scan_in_console: bool = True, 
) -> dict:
    """用二维码登录

    :param app: 扫码绑定的设备，可以是 int、str 或者 AppEnum
        app 目前发现的可用值：
            - 1,  "web",        AppEnum.web
            - 2,  "android",    AppEnum.android
            - 3,  "ios",        AppEnum.ios
            - 4,  "linux",      AppEnum.linux
            - 5,  "mac",        AppEnum.mac
            - 6,  "windows",    AppEnum.windows
            - 7,  "tv",         AppEnum.tv
            - 8,  "alipaymini", AppEnum.alipaymini
            - 9,  "wechatmini", AppEnum.wechatmini
            - 10, "qandroid",   AppEnum.qandroid
    :param scan_in_console: 是否在命令行输出二维码，否则会在默认的图片打开程序中打开

    :return: 扫码登录结果
    """
    qrcode_token = get_qrcode_token()["data"]
    qrcode = qrcode_token.pop("qrcode")
    if scan_in_console:
        try:
            from qrcode import QRCode
        except ModuleNotFoundError:
            from sys import executable
            from subprocess import run
            run([executable, "-m", "pip", "install", "qrcode"], check=True)
            from qrcode import QRCode # type: ignore
        qr = QRCode(border=1)
        qr.add_data(qrcode)
        qr.print_ascii(tty=True)
    else:
        from atexit import register
        from os import remove
        from threading import Thread
        from tempfile import NamedTemporaryFile
        qrcode_image = get_qrcode(qrcode_token["uid"])
        with NamedTemporaryFile(suffix=".png", delete=False) as f:
            f.write(qrcode_image.read())
            f.flush()
        register(lambda: remove(f.name))
        def open_qrcode():
            platform = __import__("platform").system()
            if platform == "Windows":
                from os import startfile # type: ignore
                startfile(f.name)
            elif platform == "Darwin":
                from subprocess import run
                run(["open", f.name])
            else:
                from subprocess import run
                run(["xdg-open", f.name])
        Thread(target=open_qrcode).start()
    while True:
        try:
            resp = get_qrcode_status(qrcode_token)
        except TimeoutError:
            continue
        status = resp["data"].get("status")
        if status == 0:
            print("[status=0] qrcode: waiting")
        elif status == 1:
            print("[status=1] qrcode: scanned")
        elif status == 2:
            print("[status=2] qrcode: signed in")
            break
        elif status == -1:
            raise OSError("[status=-1] qrcode: expired")
        elif status == -2:
            raise OSError("[status=-2] qrcode: canceled")
        else:
            raise OSError("qrcode: aborted with %r" % resp)
    return post_qrcode_result(qrcode_token["uid"], app)


def login_with_autoscan(cookie: str, app: str = "web") -> dict:
    headers = {"Cookie": cookie}
    token = get_qrcode_token()["data"]
    uid = token["uid"]
    # scanned
    scanned_data = load(urlopen(Request(
        "https://qrcodeapi.115.com/api/2.0/prompt.php?uid=%s" % uid, 
        headers=headers, 
    )))["data"]
    # comfirmed
    urlopen(Request(
        scanned_data["do_url"], 
        data=bytes(urlencode(scanned_data["do_params"]), "utf-8"), 
        headers=headers, 
    ))
    return post_qrcode_result(uid, app)


def alist_update_115_cookie(client: AlistClient, cookie: str):
    """更新 alist 中有关 115 的存储的 cookie
    """
    storages = client.admin_storage_list()["data"]["content"]
    for storage in storages:
        if storage["driver"] in ("115 Cloud", "115 Share"):
            addition = loads(storage["addition"])
            addition["cookie"] = cookie
            storage["addition"] = dumps(addition)
            client.admin_storage_update(storage)


def main(args):
    if args.version:
        print(".".join(map(str, __version__)))
        raise SystemExit(0)

    client = AlistClient(args.origin, args.username, args.password)
    resp = client.admin_user_get(1)
    if resp["code"] != 200:
        raise ValueError(resp)

    if args.set_cookie:
        cookie = args.set_cookie
    else:
        if args.cookie:
            resp = login_with_autoscan(args.cookie, args.app)
        else:
            resp = login_with_qrcode(args.app, scan_in_console=not args.open_qrcode)
        cookie = "; ".join("%s=%s" % t for t in resp["data"]["cookie"].items())

    alist_update_115_cookie(client, cookie)


parser.add_argument("app", nargs="?", choices=("web", "android", "ios", "linux", "mac", "windows", "tv", "alipaymini", "wechatmini", "qandroid"), default="web", help="选择一个 app 进行登录，默认为 'web'，注意：这会把已经登录的相同 app 踢下线")
parser.add_argument("-o", "--origin", default="http://localhost:5244", help="alist 服务器地址，默认 http://localhost:5244")
parser.add_argument("-u", "--username", default="", help="用户名，默认为空")
parser.add_argument("-p", "--password", default="", help="密码，默认为空")
parser.add_argument("-c", "--cookie", help="115 登录 cookie，如果提供了，就可以用这个 cookie 进行自动扫码，否则需要你用手机来手动扫码")
parser.add_argument("-ck", "--set-cookie", help="115 登录 cookie，如果指定则直接把这个 cookie 更新到 alist")
parser.add_argument("-op", "--open-qrcode", action="store_true", help="打开二维码图片，而不是在命令行输出")
parser.add_argument("-v", "--version", action="store_true", help="输出版本号")
parser.set_defaults(func=main)


if __name__ == "__main__":
    args = parser.parse_args()
    main(args)

==> python_alist-0.0.11.6.tar.gz: python_alist-0.0.11.6/alist/cmd/update_115_cookie.py <==
#!/usr/bin/env python3
# encoding: utf-8

__author__ = "ChenyangGao <https://chenyanggao.github.io>"
__all__: list[str] = []
__doc__ = "alist 更新 115 cookie"

if __name__ == "__main__":
    from argparse import ArgumentParser, RawTextHelpFormatter

    parser = ArgumentParser(description=__doc__, formatter_class=RawTextHelpFormatter)
else:
    from .init import subparsers

    parser = subparsers.add_parser("115", description=__doc__)

from enum import Enum
from http.client import HTTPResponse
from json import dumps, load, loads
from urllib.parse import urlencode
from urllib.request import urlopen, Request

from alist import __version__, AlistClient


AppEnum = Enum("AppEnum", "web, android, ios, linux, mac, windows, tv, alipaymini, wechatmini, qandroid")


def get_enum_name(val, cls):
    if isinstance(val, cls):
        return val.name
    try:
        if isinstance(val, str):
            return cls[val].name
    except KeyError:
        pass
    return cls(val).name


def get_qrcode_token() -> dict:
    """获取登录二维码，扫码可用
    GET https://qrcodeapi.115.com/api/1.0/web/1.0/token/

    :return: 扫码相关的信息，比如二维码的 uid
    """
    api = "https://qrcodeapi.115.com/api/1.0/web/1.0/token/"
    return load(urlopen(api))


def get_qrcode_status(payload) -> dict:
    """获取二维码的状态（未扫描、已扫描、已登录、已取消、已过期等）
    GET https://qrcodeapi.115.com/get/status/

    :param payload: 请求的查询参数，取自 `login_qrcode_token` 接口响应，有 3 个
        - uid:  str
        - time: int
        - sign: str

    :return: 二维码目前的扫码状态信息
    """
    api = "https://qrcodeapi.115.com/get/status/?" + urlencode(payload)
    return load(urlopen(api))


def post_qrcode_result(uid: str, app: str = "web") -> dict:
    """获取扫码登录的结果，并且绑定设备，包含 cookie
    POST https://passportapi.115.com/app/1.0/{app}/1.0/login/qrcode/

    :param uid: 二维码的 uid，取自 `login_qrcode_token` 接口响应
    :param app: 扫码绑定的设备，可以是 int、str 或者 AppEnum
        app 目前发现的可用值：
            - 1,  "web",        AppEnum.web
            - 2,  "android",    AppEnum.android
            - 3,  "ios",        AppEnum.ios
            - 4,  "linux",      AppEnum.linux
            - 5,  "mac",        AppEnum.mac
            - 6,  "windows",    AppEnum.windows
            - 7,  "tv",         AppEnum.tv
            - 8,  "alipaymini", AppEnum.alipaymini
            - 9,  "wechatmini", AppEnum.wechatmini
            - 10, "qandroid",   AppEnum.qandroid

    :return: 包含 cookie 的响应
    """
    app = get_enum_name(app, AppEnum)
    payload = {"app": app, "account": uid}
    api = "https://passportapi.115.com/app/1.0/%s/1.0/login/qrcode/" % app
    return load(urlopen(Request(api, data=urlencode(payload).encode("utf-8"), method="POST")))


def get_qrcode(uid: str) -> HTTPResponse:
    """获取二维码图片（注意不是链接）
    :return: 一个文件对象，可以读取
    """
    url = "https://qrcodeapi.115.com/api/1.0/mac/1.0/qrcode?uid=%s" % uid
    return urlopen(url)


def login_with_qrcode(
    app: str = "web", 
    scan_in_console: bool = True, 
) -> dict:
    """用二维码登录

    :param app: 扫码绑定的设备，可以是 int、str 或者 AppEnum
        app 目前发现的可用值：
            - 1,  "web",        AppEnum.web
            - 2,  "android",    AppEnum.android
            - 3,  "ios",        AppEnum.ios
            - 4,  "linux",      AppEnum.linux
            - 5,  "mac",        AppEnum.mac
            - 6,  "windows",    AppEnum.windows
            - 7,  "tv",         AppEnum.tv
            - 8,  "alipaymini", AppEnum.alipaymini
            - 9,  "wechatmini", AppEnum.wechatmini
            - 10, "qandroid",   AppEnum.qandroid
    :param scan_in_console: 是否在命令行输出二维码，否则会在默认的图片打开程序中打开

    :return: 扫码登录结果
    """
    qrcode_token = get_qrcode_token()["data"]
    qrcode = qrcode_token.pop("qrcode")
    if scan_in_console:
        try:
            from qrcode import QRCode
        except ModuleNotFoundError:
            from sys import executable
            from subprocess import run
            run([executable, "-m", "pip", "install", "qrcode"], check=True)
            from qrcode import QRCode # type: ignore
        qr = QRCode(border=1)
        qr.add_data(qrcode)
        qr.print_ascii(tty=True)
    else:
        from atexit import register
        from os import remove
        from threading import Thread
        from tempfile import NamedTemporaryFile
        qrcode_image = get_qrcode(qrcode_token["uid"])
        with NamedTemporaryFile(suffix=".png", delete=False) as f:
            f.write(qrcode_image.read())
            f.flush()
        register(lambda: remove(f.name))
        def open_qrcode():
            platform = __import__("platform").system()
            if platform == "Windows":
                from os import startfile # type: ignore
                startfile(f.name)
            elif platform == "Darwin":
                from subprocess import run
                run(["open", f.name])
            else:
                from subprocess import run
                run(["xdg-open", f.name])
        Thread(target=open_qrcode).start()
    while True:
        try:
            resp = get_qrcode_status(qrcode_token)
        except TimeoutError:
            continue
        status = resp["data"].get("status")
        if status == 0:
            print("[status=0] qrcode: waiting")
        elif status == 1:
            print("[status=1] qrcode: scanned")
        elif status == 2:
            print("[status=2] qrcode: signed in")
            break
        elif status == -1:
            raise OSError("[status=-1] qrcode: expired")
        elif status == -2:
            raise OSError("[status=-2] qrcode: canceled")
        else:
            raise OSError("qrcode: aborted with %r" % resp)
    return post_qrcode_result(qrcode_token["uid"], app)


def login_with_autoscan(cookie: str, app: str = "web") -> dict:
    headers = {"Cookie": cookie}
    token = get_qrcode_token()["data"]
    uid = token["uid"]
    # scanned
    scanned_data = load(urlopen(Request(
        "https://qrcodeapi.115.com/api/2.0/prompt.php?uid=%s" % uid, 
        headers=headers, 
    )))["data"]
    # comfirmed
    urlopen(Request(
        scanned_data["do_url"], 
        data=bytes(urlencode(scanned_data["do_params"]), "utf-8"), 
        headers=headers, 
    ))
    return post_qrcode_result(uid, app)


def alist_update_115_cookie(
    client: AlistClient, 
    cookie: str, 
    only_not_work: bool = True, 
):
    """更新 alist 中有关 115 的存储的 cookie
    """
    storages = client.admin_storage_list()["data"]["content"]
    for storage in storages:
        if storage["driver"] in ("115 Cloud", "115 Share"):
            if only_not_work and storage["status"] == "work":
                continue
            addition = loads(storage["addition"])
            addition["cookie"] = cookie
            storage["addition"] = dumps(addition)
            client.admin_storage_update(storage)


def main(args):
    if args.version:
        print(".".join(map(str, __version__)))
        raise SystemExit(0)

    client = AlistClient(args.origin, args.username, args.password)
    resp = client.admin_storage_list()
    if resp["code"] != 200:
        raise ValueError(resp)

    if args.only_not_work and all(
        storage["status"] == "work" 
        for storage in resp["data"]["content"]
        if storage["driver"] in ("115 Cloud", "115 Share")
    ):
        return

    if args.set_cookie:
        cookie = args.set_cookie
    else:
        if args.cookie:
            resp = login_with_autoscan(args.cookie, args.app)
        else:
            resp = login_with_qrcode(args.app, scan_in_console=not args.open_qrcode)
        cookie = "; ".join("%s=%s" % t for t in resp["data"]["cookie"].items())

    alist_update_115_cookie(client, cookie)


parser.add_argument("app", nargs="?", choices=("web", "android", "ios", "linux", "mac", "windows", "tv", "alipaymini", "wechatmini", "qandroid"), default="web", help="选择一个 app 进行登录，默认为 'web'，注意：这会把已经登录的相同 app 踢下线")
parser.add_argument("-o", "--origin", default="http://localhost:5244", help="alist 服务器地址，默认 http://localhost:5244")
parser.add_argument("-u", "--username", default="admin", help="用户名，默认为 admin")
parser.add_argument("-p", "--password", default="", help="密码，默认为空")
parser.add_argument("-c", "--cookie", help="115 登录 cookie，如果提供了，就可以用这个 cookie 进行自动扫码，否则需要你用手机来手动扫码")
parser.add_argument("-ck", "--set-cookie", help="115 登录 cookie，如果指定则直接把这个 cookie 更新到 alist")
parser.add_argument("-op", "--open-qrcode", action="store_true", help="打开二维码图片，而不是在命令行输出")
parser.add_argument("-on", "--only-not-work", action="store_true", help="如果 cookie 未失效则不进行更新")
parser.add_argument("-v", "--version", action="store_true", help="输出版本号")
parser.set_defaults(func=main)


if __name__ == "__main__":
    args = parser.parse_args()
    main(args)
